#version 330

#define PARTICELS 5
#define DEBUG 4
#define SKYBOX 3
#define MAX_LIGHTS 10

uniform int numLights;
uniform struct Light {
   vec4 position;
   vec4 intensities;				//a.k.a the color of the light
   float attenuationConst;
   float attenuationLin;
   float attenuationQuad;
   float ambientCoefficient;
   float coneAngle;
   vec4 coneDirection;
   mat4 depthMVP;
} allLights[MAX_LIGHTS];

layout (std140) uniform Material {
    vec4 diffuse;
    vec4 ambient;
    vec4 specular;
    vec4 emissive;
    float shininess;
    int texCount;
};

uniform vec4 dbg_color;
uniform sampler2D texUnit;
uniform sampler2D projectedTex;
uniform int mode;
uniform int makeShadow;
uniform vec3 camPos;
uniform int cellShadingAtt;
uniform int cellShadingCol;

uniform float xtrShininess;

uniform int texBool;

uniform mat4 bias;
uniform sampler2DShadow shadowMap1;
uniform sampler2DShadow shadowMap2;
uniform samplerCubeShadow shadowCubeMap;

smooth in vec3 fragNormal;
smooth in vec3 vertPos;
in vec4 shadowCoords1;
in vec4 shadowCoords2;
in vec2 TexCoord;
in vec4 projectedCoords;
in vec4 gl_FragCoord;

layout(location=0) out vec4 fragColor;
precision mediump float;

const float screenGamma = 2.2; // Assume the monitor is calibrated to the sRGB color space

vec4 test = vec4(0);

float attenuation = 1;

float calcNDCDistance(Light light){
	float f = 100.0;
	float n = 1.2;
	vec3 lightToVert = vertPos.xyz - light.position.xyz;
	vec3 AbsVec = abs(lightToVert);
	float LocalZcomp = max(AbsVec.x, max(AbsVec.y, AbsVec.z));

	float NormZComp = (f+n) / (f-n) - (2*f*n)/(f-n)/LocalZcomp;

	float distance = (NormZComp + 1) * 0.5;
	return distance;
}

vec4 ApplyLight(Light light, vec4 surfaceColor, vec3 normal, vec3 surfacePos, float specularCoefficient, int i) {
    vec3 surfaceToLight;
    attenuation = 1.0;

    if(light.position.w == 0.0) {
	
        //directional light
        surfaceToLight = normalize(-light.coneDirection.xyz);
        attenuation = 1.0; 

    } else {

        //point light
        surfaceToLight = normalize(light.position.xyz - surfacePos);
        float distanceToLight = length(light.position.xyz - surfacePos);

		float d = distanceToLight;

        float kc = light.attenuationConst;
        float kl = light.attenuationLin;
        float kq = light.attenuationQuad;

        attenuation =  ( 1.0f / (kc + kl*d + kq*d*d));


        //cone restrictions (affects attenuation)

        float lightToSurfaceAngle = degrees(acos(dot(-surfaceToLight, normalize(light.coneDirection.xyz))));
		
		float diff = light.coneAngle - lightToSurfaceAngle;

		 if(diff < 0){
            attenuation = 0.0;
        } else if (diff < 5){
			attenuation = attenuation * (diff / 5);
		}
    }

   
    
    float diffuseCoefficient = max(0.0, dot(normal, surfaceToLight));


	if (cellShadingAtt > 0){

		float x = cellShadingAtt;

		specularCoefficient = max(0.0,(ceil(specularCoefficient * x)/ x) - 0.2); 
		diffuseCoefficient =  max(0.0,(ceil(diffuseCoefficient  * x)/ x) - 0.1);
	}

	
    vec4 diffuse = vec4(diffuseCoefficient * surfaceColor.rgb * light.intensities.rgb ,surfaceColor.a);
    vec4 specular = specularCoefficient * specular * light.intensities;


	if(i == 0){
		attenuation *= (texture( shadowMap1, vec3(shadowCoords1.xy, (shadowCoords1.z)/shadowCoords1.w)));
	} else if(i == 1 ){
		attenuation *= (texture( shadowMap2, vec3(shadowCoords2.xy/shadowCoords2.w, (shadowCoords2.z)/shadowCoords2.w)));
	} else if (i == 2){
		attenuation *= texture( shadowCubeMap, vec4(-surfaceToLight, calcNDCDistance(light)));
	}
	

    return attenuation*(specular + diffuse);
}



void main() {

	float uShininess = shininess;
	if (xtrShininess > 0.0f){
		float uShininess = xtrShininess;
	}

	if (makeShadow == 0){
  
		vec3 normal = normalize(fragNormal);
		vec4 surfaceColor = diffuse;

		
  
								//TEXTURE
		test = vec4(texBool);
		if (texBool == 1){
			surfaceColor =  texture(texUnit, TexCoord).rgba;
			
		} 

		if (mode == PARTICELS){
			surfaceColor = texture(texUnit, TexCoord).rgba;
			
		}
		
		//ambient
		vec4 outAmbient = surfaceColor;
		vec4 diffuSpec = vec4(0.0,0.0,0.0,0.0);
		vec4 colorLinear = surfaceColor;
		
		vec3 projCoords = projectedCoords.xyz/projectedCoords.w;
	
		
		if (projCoords.x > 0 && projCoords.y > 0 && projCoords.z < 1){
			surfaceColor += vec4((0.8 + surfaceColor.rgb) * texture(projectedTex, projCoords.xy).rgb, surfaceColor.a);
		}
		//test = surfaceColor;
		
		
		if (mode != SKYBOX && mode != DEBUG){
			colorLinear = vec4(0,0,0,0);
			for(int i = 0; i < numLights; i++){
				Light light = allLights[i];
		
				vec3 lightDir = normalize(light.position.xyz - vertPos.xyz);
				vec3 viewDir = normalize(camPos.xyz - vertPos.xyz);
				vec3 halfDir = normalize(lightDir + viewDir);
				float specAngle = max(dot(halfDir, normal), 0.0);
				
				float specInten = pow(specAngle, uShininess);

				diffuSpec = ApplyLight(light, surfaceColor, fragNormal, vertPos, specInten, i);
				outAmbient = vec4(light.ambientCoefficient * surfaceColor.rgb * light.intensities.rgb, surfaceColor.a);

				colorLinear +=  (diffuSpec + outAmbient);
				
			} 
		}					

		
  
		if (mode == 4){
			fragColor = dbg_color;

		} else {
			colorLinear += surfaceColor * ambient + surfaceColor * emissive;						// apply gamma correction (assume ambientColor, diffuseColor and specColor
			colorLinear.a = surfaceColor.a;
	
			vec4 colorGammaCorrected = pow(colorLinear, vec4(vec3(1.0/screenGamma),1));				// use the gamma corrected color in the fragment
			fragColor = colorGammaCorrected;

			if (cellShadingCol > 0){
			
				float x = cellShadingCol;

				fragColor.x = max(0.0,(ceil(fragColor.x * x)/ x) - 0.2); 
				fragColor.y = max(0.0,(ceil(fragColor.y * x)/ x) - 0.2); 
				fragColor.z = max(0.0,(ceil(fragColor.z * x)/ x) - 0.2); 

			}
				test.a = 1;
				//fragColor = test;
		}
	
	} else if (makeShadow == 1) {
		fragColor = vec4(gl_FragCoord.zzz ,1);

	}

	//test = vec4(abs(camPos),1);


}

