#version 330

#define PARTICELS 5
#define DEBUG 4
#define SKYBOX 3

#define MAX_LIGHTS 10
uniform int numLights;
uniform struct Light {
   vec4 position;
   vec4 intensities;				//a.k.a the color of the light
   float attenuationConst;
   float attenuationLin;
   float attenuationQuad;
   float ambientCoefficient;
   float coneAngle;
   vec4 coneDirection;
   mat4 depthMVP;
   sampler2DShadow shadowMap;
} allLights[MAX_LIGHTS];

layout (location = 3) in vec3 position;
in vec2 texCoord;
in vec3 normal;

uniform int mode;


smooth out vec3 fragNormal;
out vec2 TexCoord;
out vec3 vertPos;
out vec4 shadowCoords1;
out vec4 shadowCoords2;
out vec4 projectedCoords;

uniform mat4 bias;
uniform mat4 model;
uniform mat4 VP;
uniform mat4 projectorVP;
uniform int makeShadow;
uniform vec3 camPos;
uniform vec4 diffuseC;
uniform mat4 meshModel;

layout(location = 0) in vec3 squareVertices;
layout(location = 1) in vec4 xyzs; // Position of the center of the particule and size of the square
layout(location = 2) in vec4 color; // Position of the center of the particule and size of the square

vec3 vecUp = vec3(0,0,1);


void main(){

	if (makeShadow == 0) {

		gl_Position = VP * model * meshModel *   vec4(position, 1.0);

		vec4 vertPos4 =  model * meshModel * vec4(position, 1.0);
		vertPos = vec3(vertPos4) / vertPos4.w;


		fragNormal = normalize(vec3( model * meshModel *  vec4(normal,0.0)));
		TexCoord = vec2(texCoord);

        shadowCoords1 = bias * allLights[0].depthMVP *  model * meshModel * vec4(position,1);
		shadowCoords2 = bias * allLights[1].depthMVP *  model * meshModel * vec4(position,1);
		projectedCoords = bias * projectorVP *  model * meshModel *   vec4(position,1);

		
	} else if (makeShadow == 1) {
		gl_Position =  VP *  meshModel * model * vec4(position,1);
	}


	if (mode == PARTICELS){
			vec3 particleCenter_wordspace = xyzs.xyz;
			float particleSize = xyzs.w; // because we encoded it this way.

			particleSize *=  1 - (0.7 * makeShadow);

			vec3 camToParticel = camPos - particleCenter_wordspace;
			vec3 dunno = particleCenter_wordspace - vecUp;
			vec3 vecRight = normalize(cross(camToParticel, dunno));
			vec3 camUp = normalize(cross(vecRight, camToParticel));
			vec3 vertexPosition_worldspace = 
				particleCenter_wordspace
				+ camUp * squareVertices.x * particleSize
				+ vecRight * squareVertices.y * particleSize;
			fragNormal = camToParticel;

			
			shadowCoords1 = bias * allLights[0].depthMVP  * vec4(vertexPosition_worldspace,1);
			shadowCoords2 =  bias * allLights[1].depthMVP  * vec4(vertexPosition_worldspace,1);
			projectedCoords = bias * projectorVP  *  vec4(vertexPosition_worldspace,1);
				
			vertPos = vertexPosition_worldspace;
			

			TexCoord = (squareVertices.xy+0.5);
			

			// Output position of the vertex
			gl_Position = VP * vec4(vertexPosition_worldspace , 1.0f);

		}

}